/*
 * sbtest.c
 *
 * Spaceball Library Test Program
 *
 * Version: L4.00
 * Release Date:   		11 April    1989
 * Latest modification:		13 March    1989
 *
 * Written by John Hilton, Spatial Systems Inc, August 1988
 *
 * Copyright (C) 1988 Spatial Systems Inc. All Rights Reserved.
 * Spaceball is a registered trademark of Spatial Systems Inc.
 *
 * Refer to the Spaceball Application Developer's Reference (Document
 * Number D1005) for more information on the following code.
 *
 * RESTRICTIONS
 *
 * The Spaceball Library may be used and distributed freely provided the
 * routines are used exclusively in conjunction with a Spaceball. No
 * portion may be used for other purposes without prior written permission
 * from Spatial Systems Inc. This notice must accompany any full or partial
 * copies.
 *
 * PURPOSE
 *
 * This file provides a simple test program to verify the operation of
 * Spaceball.
 */

#include    <stdio.h>
#define     Sb_ENVIRONMENT_INCLUDES
#include    "sblibry.h"

char
  terminator = '\r';

int
  redisplay = FALSE,
  quit = FALSE,
  tran_on = TRUE,
  rot_on = TRUE,
  mypressed(),
  mytranslate(),
  myrotate();

SbUint16
  b_period;

float
  tran_vec[ 3 ],
  rot_vec[ 3 ];

main(argc, argv)
int argc;
char **argv;
{
    char pkt[ 200 ];
    SbUint16 now;

    if (argc < 2) {
	fprintf( stderr, "Usage:\n\tsbtest <spaceball tty>\n" );
	exit(0);
    } else if (argc > 2)
	terminator = '\n';
    initialize_spaceball( argv[ 1 ] );
    SbRequestBallData();
#if 0
{
static SbUint16 mypulse[2] = { 20, 20 };
SbSetPulse( mypulse );
}
#endif

    printf(
"  clock  |  period     tx       ty       tz         rx       ry       rz\n"
    );
    while (!quit) {
	SbWaitForPacket( 1000 );
	SbCheckSpaceball();
	if (redisplay) {
	    display_data();
	    redisplay = FALSE;
	    SbRequestBallData();
	} else {
	    now = SbMilliseconds();
	    printf( "\r  %5u  ", now );
	    fflush( stdout );
	}
    }
    SbSetBallMode( "NN" );  /* turn off ball */
    SbClose();
    putchar( '\n' );
}

initialize_spaceball( tty_line )
char *tty_line;
{
    if (SbOpen( tty_line ) < 0) {
	printf( "Could not open Spaceball on %s\n", tty_line );
	exit( 1 );
    }
    if (SbReset( TRUE ) == -1)
	printf(
	    "Spaceball did not appear to respond on %s - continuing anyway\n",
	    tty_line
	);
    SbSetBallMode( "SSV" );     /* turn translations and rotations on */
    SbFunctions.pressed = mypressed;
    SbFunctions.translate = mytranslate;
    SbFunctions.rotate = myrotate;
}

display_data()
{
    SbUint16 now;

    switch ((tran_on ? 1 : 0) | (rot_on ? 2 : 0)) {
      case 1:
	now = SbMilliseconds();
	printf( "%c  %5u  |  %5u  %8.5f %8.5f %8.5f   %s",
	    terminator, now, b_period, tran_vec[0], tran_vec[1], tran_vec[2],
	    "                          "
	);
	break;
      case 2:
	now = SbMilliseconds();
	printf( "%c  %5u  |  %5u  %s   %8.5f %8.5f %8.5f",
	    terminator, now, b_period,
	    "                          ",
	    rot_vec[0], rot_vec[1], rot_vec[2]
	);
	break;
      case 3:
	now = SbMilliseconds();
	printf( "%c  %5u  |  %5u  %8.5f %8.5f %8.5f   %8.5f %8.5f %8.5f",
	    terminator, now, b_period, tran_vec[0], tran_vec[1], tran_vec[2],
	    rot_vec[0], rot_vec[1], rot_vec[2]
	);
    }
    fflush( stdout );
}

mypressed( keys )
SbUint16 keys;
{
    SbBeep( "cE" );
    if (keys & 1) {
	tran_on = !tran_on;
	printf( "\nfunction key #1 pressed - translations toggled %s\n",
	    tran_on ? "on" : "off"
	);
	SbSetBallMode( tran_on ? "S" : "N" );
    }
    if (keys & 2) {
	rot_on = !rot_on;
	printf( "\nfunction key #2 pressed - rotations toggled %s\n",
	    rot_on ? "on" : "off"
	);
	SbSetBallMode( rot_on ? "XS" : "XN" );
    }
    if (keys & 4) {
	printf( "\nfunction key #3 pressed\n" );
    }
    if (keys & 8) {
	printf( "\nfunction key #4 pressed - quitting\n" );
	quit = TRUE;
    }
    if (keys & 0x10) {
	printf( "\nfunction key #5 pressed\n" );
    }
    if (keys & 0x20) {
	printf( "\nfunction key #6 pressed\n" );
    }
    if (keys & 0x40) {
	printf( "\nfunction key #7 pressed\n" );
    }
    if (keys & 0x80) {
	printf( "\nfunction key #8 pressed - rezeroing the ball\n" );
	SbRezero();
    }
    if (keys & 0x100) {
	printf( "\npick button pressed\n" );
    }
}

mytranslate( period, vec )
SbUint16 period;
SbReal vec[ 3 ];
{
    b_period = period;
    memcpy( tran_vec, vec, sizeof( tran_vec ));
    redisplay = TRUE;
}

myrotate( period, vec )
SbUint16 period;
SbReal vec[ 3 ];
{
    b_period = period;
    memcpy( rot_vec, vec, sizeof( rot_vec ));
    redisplay = TRUE;
}

